--------------------------------------------------------------------------
-- SmartError.lua 
--------------------------------------------------------------------------
--[[
Author: Lyta of Malygos

Change Log:
	v1.0 
		- Initial Release.
]]--

SmartError = {
	Version = "1.0";
	DataCode = "1";
	DebugMode = nil;
	Events = {
		"NoMana",
		"OutOfRange",
		"NoLoS",
		"Moving",
		"BadTarget",
		"InCombat",
		"NotFacing",
		"CrowdControlled",
		"ObjectBusy",
		"Resurrect",
		"Summon"
	};
}

SmartErrorData = { };

function SmartError_OnLoad()
    this:RegisterEvent("VARIABLES_LOADED");
end

function SmartError_ChatPrint(str)
	DEFAULT_CHAT_FRAME:AddMessage("[SmartError] "..str, 0.25, 1.0, 0.25);
end

function SmartError_ErrorPrint(str)
	DEFAULT_CHAT_FRAME:AddMessage("[SmartError] "..str, 1.0, 0.5, 0.5);
end

function SmartError_DebugPrint(str)
	if (SmartError.DebugMode) then
		DEFAULT_CHAT_FRAME:AddMessage("[SmartError] "..str, 0.75, 1.0, 0.25);
	end
end

function SmartError_OnEvent(event)
	if (event == "VARIABLES_LOADED") then
		if (SmartErrorData.DataCode ~= SmartError.DataCode) then
			SmartError_ChatPrint(SmartError.Version..": "..SMARTERROR_NEWDATABASE);
			SmartError_FullReset();
		end
		SmartError_RenderOptions();
		SmartError_ActivateMod();
		if (SmartErrorData.Active) then
			SmartError_ChatPrint(SmartError.Version..": "..SMARTERROR_LOADED);
		else
			SmartError_ChatPrint(SmartError.Version..": "..SMARTERROR_LOADED.." (|cFFFF1111"..SMARTERROR_SUSPENDED.."|r)");
		end
	end
	if (event == "UI_ERROR_MESSAGE") then
		SmartError_DebugPrint(arg1);
		if (arg1 == ERR_SPELL_COOLDOWN ) then
			--Annoyingly spammy!  I'll have to find a different sound...
			--SmartError_PlaySound(2);
		end
		if (arg1 == ERR_SPELL_OUT_OF_RANGE and SmartErrorData.Events["OutOfRange"] == true) then
			SmartError_PlaySound(2);
		end
		if (arg1 == SPELL_FAILED_MOVING and SmartErrorData.Events["Moving"] == true) then
			SmartError_PlaySound(2);
		end
		if (arg1 == SPELL_FAILED_LINE_OF_SIGHT and SmartErrorData.Events["NoLoS"] == true) then
			SmartError_PlaySound(2);
		end
		if (SmartErrorData.Events["BadTarget"] == true) then
			if (arg1 == SPELL_FAILED_BAD_TARGETS) then
				SmartError_PlaySound(8);
			end
			if (arg1 == ERR_INVALID_ATTACK_TARGET) then
				SmartError_PlaySound(8);
			end
		end
		if (arg1 == SPELL_FAILED_AFFECTING_COMBAT and SmartErrorData.Events["InCombat"] == true) then
			SmartError_PlaySound(2);
		end
		if (arg1 == ERR_OUT_OF_MANA and SmartErrorData.Events["NoMana"] == true) then
			SmartError_PlaySound(1);
		end
		if (arg1 == ERR_OBJECT_IS_BUSY and SmartErrorData.Events["ObjectBusy"] == true) then
			SmartError_PlaySound(7);
		end
		if (SmartErrorData.Events["NotFacing"] == true) then
			if (arg1 == SPELL_FAILED_UNIT_NOT_INFRONT) then
				SmartError_PlaySound(3);
			end
			if (arg1 == ERR_BADATTACKFACING) then
				SmartError_PlaySound(3);
			end
			if (arg1 == SPELL_FAILED_NOT_BEHIND) then
				SmartError_PlaySound(3);
			end
			if (arg1 == SPELL_FAILED_UNIT_NOT_BEHIND) then
				SmartError_PlaySound(3);
			end
		end
		if (SmartErrorData.Events["CrowdControlled"] == true) then
			if (arg1 == SPELL_FAILED_SILENCED) then
				SmartError_PlaySound(4);
			end
			if (arg1 == SPELL_FAILED_STUNNED) then
				SmartError_PlaySound(4);
			end
			if (arg1 == ERR_ATTACK_PACIFIED) then
				SmartError_PlaySound(4);
			end
			if (arg1 == ERR_ATTACK_CHARMED) then
				SmartError_PlaySound(4);
			end
			if (arg1 == ERR_ATTACK_CONFUSED) then
				SmartError_PlaySound(4);
			end
			if (arg1 == ERR_ATTACK_FLEEING) then
				SmartError_PlaySound(4);
			end
			if (arg1 == "Can't attack while polymorphed.") then
				SmartError_PlaySound(4);
			end
		end
	end
	if (event == "RESURRECT_REQUEST") then
		if (SmartErrorData.Events["Resurrect"] == true) then
			SmartError_PlaySound(5);
		end
	end
	if (event == "CONFIRM_SUMMON") then
		if (SmartErrorData.Events["Summon"] == true) then
			SmartError_PlaySound(6);
		end
	end
end

function SmartError_PlaySound(iSound)
	local soundTable = {
		"Interface\\AddOns\\SmartError\\Sounds\\no_mana.wav",
		"Interface\\AddOns\\SmartError\\Sounds\\no_los.wav",
		"Interface\\AddOns\\SmartError\\Sounds\\facing.wav",
		"Interface\\AddOns\\SmartError\\Sounds\\silence.wav",
		"Interface\\AddOns\\SmartError\\Sounds\\resurrect.wav",
		"Interface\\AddOns\\SmartError\\Sounds\\summon.wav",
		"Interface\\AddOns\\SmartError\\Sounds\\objectbusy.wav",
		"Interface\\AddOns\\SmartError\\Sounds\\badtarget.wav",
	};
	PlaySoundFile(soundTable[iSound]);
end

function SmartError_FullReset()
	SmartErrorData = { 
		DataCode = SmartError.DataCode;
		Active = true;
		Events = { };
	};
	SmartError_SetDefaults();
end

function SmartError_SetDefaults()
	for index, option in pairs(SmartError.Events) do
		SmartErrorData.Events[option] = true;
	end
end

function SmartError_RenderOptions()
	local ConfigurationPanel = CreateFrame("FRAME","SmartError_MainFrame");
	ConfigurationPanel.name = "SmartError";
	InterfaceOptions_AddCategory(ConfigurationPanel);

	local EnabledButton = CreateFrame("CheckButton", "SmartError_EnabledButton", ConfigurationPanel, "ChatConfigCheckButtonTemplate");
	EnabledButton:SetPoint("TOPLEFT", 10, -15);
	EnabledButton.tooltip = "Unchecking this box will disable all sounds.";
	EnabledButton:SetChecked(SmartErrorData.Active);
	getglobal(EnabledButton:GetName().."Text"):SetText("Enable Sounds");

	SmartError_CreateEventButton("NoMana", ConfigurationPanel, 30, -45, "Out of Mana", "Alert when you don't have enough mana to cast the spell");
	SmartError_CreateEventButton("OutOfRange", ConfigurationPanel, 30, -75, "Out of Range", "Alert when you are out of range of your target (special abilities only)");
	SmartError_CreateEventButton("NoLoS", ConfigurationPanel, 30, -105, "No Line of Sight", "Alert when the attack failed because the target was not in line of sight");
	SmartError_CreateEventButton("Moving", ConfigurationPanel, 30, -135, "Moving", "Alert when the attack failed because you were moving");
	SmartError_CreateEventButton("BadTarget", ConfigurationPanel, 30, -165, "Bad Target", "Alert that you're attacking an invalid target");
	SmartError_CreateEventButton("InCombat", ConfigurationPanel, 30, -195, "In Combat", "Alert when your ability failed because you were in combat");
	SmartError_CreateEventButton("NotFacing", ConfigurationPanel, 30, -225, "Wrong Direction", "Alert when your ability failed because you were either not facing the target or you're facing the wrong side of the target");
	SmartError_CreateEventButton("CrowdControlled", ConfigurationPanel, 30, -255, "Crowd Controlled (Stunned/Silenced)", "Alert when your ability failed because you're crowd controlled (stunned, silenced, feared, etc.)");
	SmartError_CreateEventButton("ObjectBusy", ConfigurationPanel, 30, -285, "Object is Busy", "Alert when you fail to pick up an item because the object was busy");
	SmartError_CreateEventButton("Resurrect", ConfigurationPanel, 30, -315, "Resurrected", "Alert when prompted to accept a resurrection");
	SmartError_CreateEventButton("Summon", ConfigurationPanel, 30, -345, "Summoned", "Alert when prompted to accept a summon");

	local ActivateAllButton = CreateFrame("Button", "SmartError_ActivateAllButton", ConfigurationPanel, "OptionsButtonTemplate");
	ActivateAllButton:SetPoint("TOPLEFT", 10, -375);
	ActivateAllButton.tooltip = "Click this button to activate all event sounds.";
	ActivateAllButton:SetScript("OnClick",SmartError_SetEventOptionAll);
	getglobal(ActivateAllButton:GetName().."Text"):SetText("Check All");

	local DeactivateAllButton = CreateFrame("Button", "SmartError_DeactivateAllButton", ConfigurationPanel, "OptionsButtonTemplate");
	DeactivateAllButton:SetPoint("TOPLEFT", 120, -375);
	DeactivateAllButton.tooltip = "Click this button to deactivate all event sounds.";
	DeactivateAllButton:SetScript("OnClick",SmartError_SetEventOptionNone);
	getglobal(DeactivateAllButton:GetName().."Text"):SetText("Uncheck All");

	ConfigurationPanel.okay = 
		function (self)
			SmartError_Option_Active(EnabledButton:GetChecked());
			for index, option in pairs(SmartError.Events) do
				SmartError_SetEventOption(option, getglobal("SmartError_EventButton_"..option):GetChecked())
			end
		end
	ConfigurationPanel.cancel = 
		function (self)
			EnabledButton:SetChecked(SmartErrorData.Active);
			for index, option in pairs(SmartError.Events) do
				SmartError_SetEventOption(option, SmartErrorData.Events[option])
			end
		end
	ConfigurationPanel.default = 
		function (self)
			SmartError_SetDefaults();
			SmartError_Option_Active(true);
			for index, option in pairs(SmartError.Events) do
				SmartError_SetEventOption(option, SmartErrorData.Events[option])
			end
		end
end

function SmartError_Option_Active(state)
	if (state) then
		SmartErrorData.Active = true;
	else
		SmartErrorData.Active = nil;
	end
	getglobal("SmartError_EnabledButton"):SetChecked(state);
	SmartError_ActivateMod();
end

function SmartError_ActivateMod()
	if (SmartErrorData.Active) then
		SmartErrorFrame:RegisterEvent("UI_ERROR_MESSAGE");
		SmartErrorFrame:RegisterEvent("RESURRECT_REQUEST");
		SmartErrorFrame:RegisterEvent("CONFIRM_SUMMON");
	else
		SmartErrorFrame:UnregisterEvent("UI_ERROR_MESSAGE");
		SmartErrorFrame:UnregisterEvent("RESURRECT_REQUEST");
		SmartErrorFrame:UnregisterEvent("CONFIRM_SUMMON");
	end	
end

function SmartError_SetEventOption(option, state)
	if (state) then
		SmartErrorData.Events[option] = true;
	else
		SmartErrorData.Events[option] = nil;
	end
	getglobal("SmartError_EventButton_"..option):SetChecked(state);
end

function SmartError_SetEventOptionAll()
	for index, option in pairs(SmartError.Events) do
		getglobal("SmartError_EventButton_"..option):SetChecked(true);
	end
end

function SmartError_SetEventOptionNone()
	for index, option in pairs(SmartError.Events) do
		getglobal("SmartError_EventButton_"..option):SetChecked(nil);
	end
end

function SmartError_CreateEventButton(optionCodeName, configurationPanel, coordX, coordY, optionName, optionDescription)
	local button = CreateFrame("CheckButton", "SmartError_EventButton_"..optionCodeName, configurationPanel, "ChatConfigCheckButtonTemplate");
	button:SetPoint("TOPLEFT", coordX, coordY);
	button.tooltip = optionDescription;
	button:SetChecked(SmartErrorData.Events[optionCodeName]);
	getglobal(button:GetName().."Text"):SetText(optionName);
end
